import os, json, glob, importlib, traceback

os.environ["OMP_NUM_THREADS"] = "1"  # Prevents crash on vms / servers
from flask import Flask
from flask_login import LoginManager
from flask_limiter import Limiter
from flask_limiter.util import get_remote_address
from flask_debugtoolbar import DebugToolbarExtension

import config

app = Flask(__name__)

# HOLY SHIT THIS QUARTERS SUBSEQUENT PAGE LOAD TIMES FOR STATIC PAGES
app.jinja_env.cache = {}
# IT ONLY INCREASES RAM USAGE BY 50MB OR SO ON SMALLER SITES
# ON BIGGER SITES IT'S WORTH IT FOR THE REDUCTION ON CPU LOAD
# AS IT REDUCES SERVER LOAD DRAMATICALLY
# DO IT ALMOST ALWAYS

app.debug = config.debug
if not os.path.isfile("secret.key"):
    with open("secret.key", "w+") as f:
        f.write(str(os.urandom(24).hex()))
with open("secret.key") as k:
    SECRET_KEY = k.read()
app.config["MAX_CONTENT_LENGTH"] = (
    16 * 1024 * 1024
)  # 16mb max upload is reasonable I think
app.config["SECRET_KEY"] = SECRET_KEY
app.config["DATABASE"] = "users.db"
login_manager = LoginManager(app)
login_manager.login_view = "admin.login"
login_manager.init_app(app)
# limiter = Limiter(
#     app, key_func=get_remote_address, default_limits=["10000 per day", "2000 per hour"]
# )

DebugToolbarExtension(app)
application = app

from app import routes  # Initializes many routes

# load all app modules in app/modules and register blueprints
def load_app_modules():
    def get_modules(modules_path):
        mods = []
        print(f"Searching for modules at - {modules_path}")
        for m in glob.glob(os.path.join(modules_path, "*")):
            if (
                os.path.isfile(m)
                and m.endswith(".py")
                and not m.endswith("__init__.py")
            ):
                mods.append(m)
            elif os.path.isfile(os.path.join(m, "module.py")):
                mods.append(os.path.join(m, "module.py"))
        print(f"Found {len(mods)} modules:")
        print(json.dumps(mods, indent=2))
        return mods

    print("\n# Loading app modules.")
    modules = []
    modules_paths = ("app/modules",)
    # plugins_paths.extend(plugins_path_list)
    for path in modules_paths:
        modules.extend(get_modules(path))

    for m in modules:
        try:
            print(f"Loading module at {m}")
            m = m[:-3].replace("/", ".").replace("\\", ".")
            importlib.import_module(m).register(app)
            print(f"Loaded {m}")
        except Exception as e:
            print(f"Exception loading module {m} - {e}")
            traceback.print_exc()

    print("# Done loading modules.\n")


load_app_modules()
